@@ -1,14 +1,13 @@
+
+
 class C:
-    # 一度に受け取れる最大データ量(5MBytes)
-    MAX_RECEIVE_SIZE = 5000 * 1000
+    # 一度に受け取れる最大データ量(50MBytes)
+    MAX_RECEIVE_SIZE = 50 * 1000 * 1000
 
     # type
     T_SERVER = 'type/server'
     T_CLIENT = 'type/client'
 
-    # Master key
-    MASTER_KEYS = []
-
 
 class V:
     # path
@@ -37,3 +36,11 @@ class Debug:
 
 class PeerToPeerError(Exception):
     pass
+
+
+__all__ = [
+    "C",
+    "V",
+    "Debug",
+    "PeerToPeerError",
+]
